const { template, types, traverse } = require("@babel/core");
const { addComment } = require("@babel/types");

/**
 * 代码片段
 */
class CodeSnippet {
    constructor(ast) {
        this.ast = ast;
    }

    removeProperty(className, propertyKey) {
        traverse(this.ast, {
            ClassDeclaration: (path) => {
                if (path.node.id.name === className) {
                    path.node.body.body = path.node.body.body.filter(
                        (item) =>
                            types.isClassProperty(item) &&
                            item.key.name !== propertyKey
                    );
                }
            },
        });
    }

    setProperty(className, propertyKey, propertyValue = types.nullLiteral()) {
        traverse(this.ast, {
            ClassDeclaration: (path) => {
                if (path.node.id.name === className) {
                    let property = path.node.body.body.find(
                        (item) =>
                            types.isClassProperty(item) &&
                            item.key.name === propertyKey
                    );
                    if (!property) {
                        path.node.body.body.push(
                            this.getProperty(propertyKey, propertyValue)
                        );
                    }
                }
            },
        });
    }

    getProperty(key, value) {
        let path = types.classProperty(types.identifier(key), value);
        addComment(path, "leading", "*\n   * 属性\n   ");
        return path;
    }



    setMethod(className, methodName) {
        traverse(this.ast, {
            ClassDeclaration: (path) => {
                if (path.node.id.name === className) {
                    let method = path.node.body.body.find(
                        (item) =>
                            types.isClassMethod(item) &&
                            item.key.name === methodName
                    );
                    if (!method) {
                        path.node.body.body.push(this.getMethod(methodName));
                    }
                }
            },
        });
    }

    getClass(className) {
        return template(`
            class %%className%% {}
        `)({
            className: types.identifier(className),
        });
    }

    getMethod(name) {
        let path = types.classMethod(
            "method",
            types.identifier(name),
            [],
            types.blockStatement([])
        );
        addComment(path, "trailing", "*\n   * 函数\n   ");
        return path;
    }
}

module.exports = CodeSnippet;
