const { traverse, types } = require("@babel/core");
const ReactTemplate = require("../React/Template");
const AngularTemplate = require("../Angular/Template");
const generate = require("@babel/generator").default;

class Template {
    constructor(ast) {
        this.ast = ast;
        this.reactTemplate = new ReactTemplate(this.ast);
        this.angularTemplate = new AngularTemplate();
        this.transformStyle();
        this.transformClassName();
        this.transformLoop();
        this.transformChildren();
        this.transformIf();
        this.transformEvent();
        this.transformAttribute();
        this.transformVariable();
    }

    transformStyle() {
        this.reactTemplate.getStyles().forEach((style) => {
            style
                .getPath()
                .replaceWith(this.angularTemplate.getStyle(style.getValue()));
        });
    }

    transformVariable() {
        this.reactTemplate.getVars().forEach((variable) => {
            variable
                .getPath()
                .replaceWith(this.angularTemplate.getVar(variable.getName()));
        });
    }

    transformAttribute() {
        this.reactTemplate.getAttributes().forEach((attribute) => {
            attribute
                .getPath()
                .replaceWith(
                    this.angularTemplate.getAttribute(
                        attribute.getName(),
                        attribute.getValue(),
                        attribute.getElementName()
                    )
                );
        });
    }

    transformEvent() {
        this.reactTemplate.getEvents().forEach((event) => {
            event
                .getPath()
                .replaceWith(
                    this.angularTemplate.getEvent(
                        event.getEventName(),
                        event.getElementName(),
                        event.getFunctionName(),
                        event.getFunctionParams()
                    )
                );
        });
    }

    transformIf() {
        this.reactTemplate.getIfs().forEach((item) => {
            item.getPath().replaceWith(
                this.angularTemplate.getIf(
                    item.getElementName(),
                    item.getCondition(),
                    item.getAttributes(),
                    item.getChildren()
                )
            );
        });
    }

    transformChildren() {
        let children = this.reactTemplate.getChildren();
        if (children) {
            this.angularTemplate.getChildren(children.getPath());
            // children.getPath().replaceWith(this.angularTemplate.getChildren());
        }
    }

    transformClassName() {
        this.reactTemplate.getClassNames().forEach((className) => {
            className
                .getPath()
                .replaceWith(
                    this.angularTemplate.getClassName(className.getValue())
                );
        });
    }

    transformLoop() {
        this.reactTemplate.getLoops().forEach((loop) => {
            loop.getPath().replaceWith(
                this.angularTemplate.getLoop(
                    loop.getElementName(),
                    loop.getItem(),
                    loop.getItems(),
                    loop.getAttributes(),
                    loop.getChildren(),
                    loop.getKey()
                )
            );
        });
    }

    getCode() {
        return generate(this.ast)
            .code.replace(/;$/, "")
            .replace(/([^=])"\{\{(.*?)\}\}"/g, "$1{{$2}}");
    }
}

module.exports = Template;
