const traverse = require("@babel/traverse").default;
const t = require("@babel/types");
const utils = require("./utils");

module.exports = {
  scriptTraverse(ast) {
    traverse(ast, {
      AssignmentExpression(path) {
        utils.toSetterMethod(path, "setData");
      },
      ObjectProperty(path) {
        if (t.isIdentifier(path.node.value, { name: "Function" })) {
          path.remove();
        }
      },
      MemberExpression(path) {
        if (
          t.isIdentifier(path.node.property, { name: "props" }) ||
          t.isIdentifier(path.node.property, { name: "state" })
        ) {
          path.node.property.name = "data";
        }
      },
      ClassProperty(path) {
        if (t.isIdentifier(path.node.key, { name: "props" })) {
          path.node.key.name = "properties";
        }
        if (t.isIdentifier(path.node.key, { name: "state" })) {
          path.node.key.name = "data";
        }
      },
      ClassBody: {
        enter(path) {
          path.node.body.push(
            t.classProperty(t.identifier("lifetimes"), t.objectExpression([]))
          );
          path.node.body.push(
            t.classProperty(t.identifier("methods"), t.objectExpression([]))
          );
        },
      },
      ClassMethod(path) {
        if (
          path.node.key.name === "created" ||
          path.node.key.name === "attached"
        ) {
          path
            .getSibling(path.container.length - 2)
            .node.value.properties.push(
              t.objectMethod(
                "method",
                t.identifier(path.node.key.name),
                [],
                t.blockStatement(path.node.body.body)
              )
            );
        } else {
          path
            .getSibling(path.container.length - 1)
            .node.value.properties.push(
              t.objectMethod(
                "method",
                t.identifier(path.node.key.name),
                [],
                t.blockStatement(path.node.body.body)
              )
            );
        }
        path.remove();
      },
      ClassDeclaration: {
        exit(path) {
          let childrens = path.node.body.body;
          childrens = childrens.map(function (item) {
            let key = item.key.name;
            let value = item.value;
            if (t.isClassProperty(item)) {
              return t.objectProperty(t.identifier(key), value);
            } else if (t.isClassMethod(item)) {
              return t.objectMethod("method", t.identifier(key), [], item.body);
            } else {
              return item;
            }
          });
          path.replaceWith(
            t.callExpression(t.identifier("Component"), [
              t.objectExpression(childrens),
            ])
          );
        },
      },
    });
  },
  viewTraverse(ast) {
    traverse(ast, {
      JSXExpressionContainer(path) {
        utils.toDoubleBracket(path);
      },
      JSXIdentifier(path) {
        if (t.isJSXIdentifier(path.node, { name: "div" })) {
          path.node.name = "view";
        }
        if (t.isJSXIdentifier(path.node, { name: "img" })) {
          path.node.name = "image";
        }
      },
    });
  },
};
