const { template, types } = require("@babel/core");
const Path = require("./Path");

class ObjectExpression extends Path {

    constructor(path) {
        super(path);
    }

    element = [];

    setProperty(name, value) {
        if (typeof value === "string") {
            value = types.stringLiteral(value);
        }
        if (typeof value === "number") {
            value = types.numericLiteral(value);
        }
        if (typeof value === "boolean") {
            value = types.booleanLiteral(value);
        }
        if (Array.isArray(value)) {
            value = types.objectExpression(value);
        }
        this.element.push(types.objectProperty(types.identifier(name), value));
    }

    setMethod(name, param = [], body = []) {
        this.element.push(
            types.objectProperty(
                types.identifier(name),
                types.functionExpression(
                    null,
                    param.map((item) => types.identifier(item)),
                    types.isBlockStatement(body)
                        ? body
                        : types.blockStatement(body)
                )
            )
        );
    }

    getNode() {
        return types.objectExpression(this.element);
    }
}

module.exports = ObjectExpression;
